#!/usr/bin/env python
# -*- coding: utf-8 -*-


class OauthException(Exception):

    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class AuthorizeException(OauthException):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .oauth2 import Oauth2


class Weibo(Oauth2):

    NAME = 'Weibo'
    AUTHORIZATION_URL = 'https://api.weibo.com/oauth2/authorize'
    ACCESS_TOKEN_URL = 'https://api.weibo.com/oauth2/access_token'
    GET_USERINFO_URL = 'https://api.weibo.com/2/users/show.json'

    def __init__(self):
        super(Weibo, self).__init__()

    def parse_token_response(self, response):
        self._access_token = response.get('access_token', '')
        self._expires_in = response.get('expires_in', '')
        self._uid = response.get('uid', '')

    def get_user_info_params(self):
        return {
            'access_token': self._access_token,
            'uid': self._uid
        }

    def parse_user_info(self, response):
        return {
            'uid': response.get('idstr'),
            'name': response.get('name'),
            'avatar': response.get('profile_image_url'),
            'raw': response
        }



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .oauth2 import Oauth2


class Github(Oauth2):

    NAME = 'Github'
    AUTHORIZATION_URL = 'https://github.com/login/oauth/authorize'
    ACCESS_TOKEN_URL = 'https://github.com/login/oauth/access_token'
    GET_USERINFO_URL = 'https://api.github.com/user'

    def __init__(self):
        super(Github, self).__init__()

    def parse_user_info(self, response):
        return {
            'uid': str(response.get('id')),
            'name': response.get('name'),
            'avatar': response.get('avatar_url'),
            'raw': response
        }



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import requests
from collections import OrderedDict

from .exceptions import AuthorizeException


class Base(object):
    NAME = ''
    AUTHORIZATION_URL = ''
    ACCESS_TOKEN_URL = ''
    GET_USERINFO_URL = ''

    _config = {}

    _access_token = ''
    _expires_in = ''
    _refresh_token = ''

    state = ''

    def init(self, config):
        self._config = config

    def get_config(self):
        return self._config

    def get_login_url(self):
        raise NotImplementedError('Must implement in subclass')

    def get_user_info(self):
        raise NotImplementedError('Must implement in subclass')

    def set_access_token(self):
        raise NotImplementedError('Must implement in subclass')

    def access_resource(self):
        raise NotImplementedError('Must implement in subclass')

    def _get(self, request):
        r = None
        try:
            r = requests.get(request.get('url'),
                             params=request.get('params'),
                             headers={'Accept': 'application/json'},
                             auth=request.get('auth'))
        except requests.ConnectionError as e:
            raise AuthorizeException('Connection error: {0}'.format(e))
        else:
            if r.status_code != 200:
                raise AuthorizeException(
                    'Authorization failed: {0}'.format(r.content))

        try:
            return r.json()
        except ValueError as e:
            return r.content

    def _post(self, request):
        r = None
        try:
            r = requests.post(request.get('url'),
                              params=request.get('params'),
                              data=request.get('data'),
                              headers={'Accept': 'application/json'},
                              auth=request.get('auth'))
        except requests.ConnectionError as e:
            raise AuthorizeException('Connection error: {0}'.format(e))
        else:
            if r.status_code != 200:
                raise AuthorizeException(
                    'Authorization failed: {0}'.format(r.content))

        try:
            return r.json()
        except ValueError as e:
            return r.content

    def _request(self, request):
        r = None
        try:
            r = requests.request(
                method=request.get('method', 'GET'),
                url=request.get('url'),
                params=request.get('params'),
                data=request.get('data'),
                headers={'Accept': 'application/json'},
                auth=request.get('auth')
            )
        except requests.ConnectionError as e:
            raise AuthorizeException('Connection error: {0}'.format(e))
        else:
            if r.status_code != 200:
                raise AuthorizeException(
                    'Authorization failed: {0}'.format(r.content))

        try:
            return r.json()
        except ValueError as e:
            return r.content

    def _build_request_uri(self, request):
        d = request.get('params')
        o_dict = OrderedDict(
            sorted(d.items(), key=lambda x: x[0], reverse=False))
        params = '&'.join(
            ['{0}={1}'.format(x, y)
             for x, y in o_dict.items()])
        return '{0}?{1}'.format(request.get('url'), params)

    def _query_to_dict(self, query):
        return {x.split('=')[0]: x.split('=')[1]
                for x in query.split('&')}

    def _check_config(self):
        if not self._config:
            raise AuthorizeException('Oauth client config not set')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .oauth import Oauth


class Twitter(Oauth):

    NAME = 'Twitter'
    REQUEST_TOKEN_URL = 'https://api.twitter.com/oauth/request_token'
    AUTHENTICATE_URL = 'https://api.twitter.com/oauth/authenticate'
    ACCESS_TOKEN_URL = 'https://api.twitter.com/oauth/access_token'
    GET_USERINFO_URL = 'https://api.twitter.com/1.1/account/verify_credentials.json'

    def __init__(self):
        super(Twitter, self).__init__()

    def parse_user_info(self, response):
        return {
            'uid': response.get('id_str'),
            'name': response.get('name'),
            'avatar': response.get('profile_image_url_https'),
            'raw': response
        }



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .oauth2 import Oauth2


class Facebook(Oauth2):

    NAME = 'Facebook'
    AUTHORIZATION_URL = 'https://www.facebook.com/dialog/oauth'
    ACCESS_TOKEN_URL = 'https://graph.facebook.com/oauth/access_token'
    GET_USERINFO_URL = 'https://graph.facebook.com/me?fields=id,name,email,picture'

    def __init__(self):
        super(Facebook, self).__init__()

    def parse_token_response(self, response):
        r = self._query_to_dict(response)
        self._access_token = r.get('access_token', '')
        self._expires_in = r.get('expires', '')

    def parse_user_info(self, response):
        return {
            'uid': response.get('id'),
            'name': response.get('name'),
            'avatar': response.get('picture', {}).get('data', {}).get('url'),
            'raw': response
        }



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .base import Base
from .exceptions import OauthException


class Oauth2(Base):

    def __init__(self):
        super(Oauth2, self).__init__()

    def get_login_url(self, state=None):
        self._check_config()

        params = {
            'client_id': self._config['client_id'],
            'redirect_uri': self._config['redirect_uri'],
            'response_type': 'code'
        }

        if self._config['scope']:
            params['scope'] = self._config['scope']

        if state:
            params['state'] = state

        return self._build_request_uri({
            'url': self.AUTHORIZATION_URL,
            'params': params
        })

    def get_access_token(self, refresh_token=None):
        """access_token, or use refresh_token update access_token."""
        self._check_config()

        if refresh_token:
            self._refresh_token = refresh_token
            self._get_access_token({}, 'refresh_token')

        return {
            'access_token': self._access_token,
            'expires_in': self._expires_in,
            'refresh_token': self._refresh_token
        }

    def get_user_info(self, query):
        self._check_config()

        params = query
        if not type(query) is dict:
            params = self._query_to_dict(query)

        self._check_response(params)
        self._get_access_token(params)
        response = self._get_user_info()
        return self.parse_user_info(response)

    def parse_token_response(self, response):
        self._access_token = response.get('access_token', '')
        self._expires_in = response.get('expires_in', '')
        self._refresh_token = response.get('refresh_token', '')

    def get_user_info_params(self):
        return {
            'access_token': self._access_token
        }

    def parse_user_info(self, response):
        raise NotImplementedError('Must implement in subclass')

    def set_access_token(self, access_token):
        self._access_token = access_token['access_token']
        self._expires_in = access_token.get('expires_in')

    def access_resource(self, method, url, params=None, data=None):
        self._check_config()

        if not params:
            params = {}

        params['access_token'] = self._access_token

        r = None
        if method.lower() == 'get':
            r = self._get({
                'url': url,
                'params': params
            })
        elif method.lower() == 'post':
            r = self._post({
                'url': url,
                'params': params,
                'data': data
            })
        else:
            r = self._request({
                'method': method,
                'url': url,
                'params': params,
                'data': data
            })

        return r

    def _check_response(self, params):
        if params.get('error', ''):
            raise OauthException(
                'Authorization failed: {0}'.format(params['error']))

        self.state = params.get('state', '')

    def _get_access_token(self, params, grant_type='authorization_code'):
        params['client_id'] = self._config['client_id']
        params['client_secret'] = self._config['client_secret']
        params['redirect_uri'] = self._config['redirect_uri']
        params['grant_type'] = grant_type

        if grant_type != 'authorization_code':
            params['refresh_token'] = self._refresh_token

        r = self._post({
            'url': self.ACCESS_TOKEN_URL,
            'data': params
        })

        self.parse_token_response(r)

    def _get_user_info(self):
        params = self.get_user_info_params()
        return self._get({
            'url': self.GET_USERINFO_URL,
            'params': params
        })



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import os
import json


def get_configs():
    f = os.path.join(sys.path[0], 'oauth2py.json')
    if os.path.isfile(f):
        try:
            with open(f) as data:
                return json.load(data)
        except Exception as e:
            raise 'could not load configs: {0}!'.format(e)


class OauthClient(object):
    __configs = get_configs()

    def __init__(self):
        super(OauthClient, self).__init__()

    @classmethod
    def load(cls, name):
        module = OauthClient.get_class('oauth2py.{0}.{1}'.format(
            name.lower(), name.lower().capitalize()))
        client = module()

        if cls.__configs:
            cfg = next((c for c in cls.__configs
                        if c['name'].lower() == name.lower()), None)
            if cfg:
                client.init(cfg)

        return client

    @classmethod
    def reload_configs(cls):
        cls.__configs = get_configs()

    @staticmethod
    def get_class(kls):
        parts = kls.split('.')
        module = '.'.join(parts[:-1])
        m = __import__(module)
        for comp in parts[1:]:
            m = getattr(m, comp)
        return m



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json
from .oauth2 import Oauth2


class Qq(Oauth2):

    NAME = 'QQ'
    AUTHORIZATION_URL = 'https://graph.qq.com/oauth2.0/authorize'
    ACCESS_TOKEN_URL = 'https://graph.qq.com/oauth2.0/token'
    GET_USERINFO_URL = 'https://graph.qq.com/oauth2.0/me'

    def __init__(self):
        super(Qq, self).__init__()

    def parse_token_response(self, response):
        d = self._query_to_dict(response)
        self._access_token = d.get('access_token', '')
        self._expires_in = d.get('expires_in', '')
        self._refresh_token = d.get('refresh_token', '')

    def parse_user_info(self, response):
        r = response[response.index('(') + 1: response.rindex(')')]
        d = json.loads(r)
        uid = d.get('openid', '')
        detail = self.access_resource(
            'GET',
            'https://graph.qq.com/user/get_user_info',
            params={
                'oauth_consumer_key': self._config['client_id'],
                'openid': uid
            })
        return {
            'uid': uid,
            'name': detail.get('nickname'),
            'avatar': detail.get('figureurl_qq_1'),
            'raw': detail
        }



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from requests_oauthlib import OAuth1

from .base import Base
from .exceptions import AuthorizeException


class Oauth(Base):

    def __init__(self):
        super(Oauth, self).__init__()

    def get_login_url(self):
        self._check_config()
        self._request_token = self._get_request_token()
        return '{0}?oauth_token={1}'.format(
            self.AUTHENTICATE_URL,
            self._request_token['oauth_token']
        )

    def get_request_token(self):
        return self._request_token

    def get_access_token(self, verifier=None):
        if verifier:
            oauth = OAuth1(
                client_key=self._config['client_id'],
                client_secret=self._config['client_secret'],
                callback_uri=self._config['redirect_uri'],
                resource_owner_key=self._request_token['oauth_token'],
                resource_owner_secret=self._request_token[
                    'oauth_token_secret'],
                verifier=verifier
            )

            r = self._post({
                'url': self.ACCESS_TOKEN_URL,
                'auth': oauth
            })
            q = self._query_to_dict(r)
            self._access_token = q['oauth_token']
            self._access_token_secret = q['oauth_token_secret']

        return {
            'access_token': self._access_token,
            'access_token_secret': self._access_token_secret
        }

    def get_user_info(self, query, request_token=None):
        params = query
        if not type(query) is dict:
            params = self._query_to_dict(query)

        if request_token:
            self._request_token = request_token

        self._check_response(params)
        self._get_access_token(params)

        oauth = OAuth1(
            client_key=self._config['client_id'],
            client_secret=self._config['client_secret'],
            callback_uri=self._config['redirect_uri'],
            resource_owner_key=self._access_token,
            resource_owner_secret=self._access_token_secret
        )

        r = self._get({
            'url': self.GET_USERINFO_URL,
            'auth': oauth
        })
        return self.parse_user_info(r)

    def parse_user_info(self, response):
        raise NotImplementedError('Must implement in subclass')

    def set_access_token(self, access_token):
        self._access_token = access_token['access_token']
        self._access_token_secret = access_token['access_token_secret']

    def access_resource(self, method, url, params=None, data=None):
        self._check_config()

        oauth = OAuth1(
            client_key=self._config['client_id'],
            client_secret=self._config['client_secret'],
            callback_uri=self._config['redirect_uri'],
            resource_owner_key=self._access_token,
            resource_owner_secret=self._access_token_secret
        )

        r = None
        if method.lower() == 'get':
            r = self._get({
                'url': url,
                'params': params,
                'auth': oauth
            })
        elif method.lower() == 'post':
            r = self._post({
                'url': url,
                'params': params,
                'data': data,
                'auth': oauth
            })
        else:
            r = self._request({
                'method': method,
                'url': url,
                'params': params,
                'data': data,
                'auth': oauth
            })

        return r

    def _check_response(self, params):
        if params.get('error', ''):
            raise AuthorizeException(
                'Authorization failed: {0}'.format(params['error']))

    def _get_request_token(self):
        oauth = OAuth1(
            client_key=self._config['client_id'],
            client_secret=self._config['client_secret'],
            callback_uri=self._config['redirect_uri'],
        )
        r = self._post({
            'url': self.REQUEST_TOKEN_URL,
            'auth': oauth
        })
        return self._query_to_dict(r)

    def _get_access_token(self, params):
        return self.get_access_token(params['oauth_verifier'])



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import codecs
from setuptools import setup


def long_description():
    with codecs.open('README.rst', encoding='utf-8') as f:
        long_description = f.read()

config = {
    'name': 'oauth2py',
    'version': '1.1.5',
    'description': 'a simple, lightweight oauth client',
    'author': 'caoyue',
    'author_email': 'i@caoyue.me',
    'url': 'https://github.com/caoyue/oauth2py',
    'download_url': 'https://github.com/caoyue/oauth2py',
    'license': 'http://www.apache.org/licenses/LICENSE-2.0',
    'install_requires': [''],
    'packages': ['oauth2py'],
    'classifiers': [
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ],
    'long_description': long_description(),
}

setup(**config)
